@@ -14,6 +14,7 @@ use deno_core::OpState;
 use deno_tls::rustls::RootCertStore;
 use std::cell::RefCell;
 use std::path::Path;
+use std::path::PathBuf;
 use std::rc::Rc;
 
 pub trait NetPermissions {
@@ -60,6 +61,10 @@ pub fn check_unstable2(state: &Rc<RefCell<OpState>>, api_name: &str) {
   state.borrow::<UnstableChecker>().check_unstable(api_name)
 }
 
+pub fn get_declaration() -> PathBuf {
+  PathBuf::from(env!("CARGO_MANIFEST_DIR")).join("lib.deno_net.d.ts")
+}
+
 #[derive(Clone)]
 pub struct DefaultTlsOptions {
   pub root_cert_store: Option<RootCertStore>,
